@@ -13,7 +13,7 @@ extern int device_type(const char *ifname);
 extern short device_get_flags(const char *ifname);
 extern void device_set_flags(const char *ifname, const short flags);
 extern int device_up_and_running(const char *ifname);
-extern u32 device_bitrate(const char *ifname);
+extern uint32_t device_bitrate(const char *ifname);
 extern short device_enter_promiscuous_mode(const char *ifname);
 extern void device_leave_promiscuous_mode(const char *ifname, short oldflags);
 extern const char *device_type2str(uint16_t type);
