@@ -10,14 +10,25 @@ enum ip4_field {
 	IP4_LEN,
 	IP4_ID,
 	IP4_FLAGS,
+	IP4_MF,
+	IP4_DF,
 	IP4_FRAG_OFFS,
 	IP4_TTL,
 	IP4_PROTO,
 	IP4_CSUM,
 	IP4_SADDR,
 	IP4_DADDR,
-	IP4_DF,
-	IP4_MF,
+};
+
+enum ip6_field {
+	IP6_VER,
+	IP6_CLASS,
+	IP6_FLOW_LBL,
+	IP6_LEN,
+	IP6_NEXT_HDR,
+	IP6_HOP_LIMIT,
+	IP6_SADDR,
+	IP6_DADDR,
 };
 
 extern void protos_l3_init(void);
